@@ -1,14 +1,14 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef _CM_CALLBACK_H_ENV__
-#define _CM_CALLBACK_H_ENV__ 1
+#ifndef OPENAFS_WINNT_AFSD_CM_CALLBACK_H
+#define OPENAFS_WINNT_AFSD_CM_CALLBACK_H 1
 
 #include <osi.h>
 
@@ -57,7 +57,7 @@ extern int cm_HaveCallback(struct cm_scache *);
 extern void cm_StartCallbackGrantingCall(struct cm_scache *, cm_callbackRequest_t *);
 
 extern void cm_EndCallbackGrantingCall(struct cm_scache *, cm_callbackRequest_t *,
-	struct AFSCallBack *, long);
+	struct AFSCallBack *, struct AFSVolSync *, long);
 
 extern long cm_GetCallback(struct cm_scache *, struct cm_user *,
 	struct cm_req * reqp, long flags);
@@ -68,4 +68,15 @@ extern osi_rwlock_t cm_callbackLock;
 
 extern void cm_CallbackNotifyChange(cm_scache_t *scp);
 
-#endif /*  _CM_CALLBACK_H_ENV__ */
+extern void cm_GiveUpAllCallbacks(cm_server_t *tsp, afs_int32 markDown);
+
+extern void cm_GiveUpAllCallbacksAllServers(afs_int32 markDown);
+
+extern void cm_GiveUpAllCallbacksAllServersMulti(afs_int32 markDown);
+
+extern afs_int32 cm_OfflineROIsValid;
+
+extern afs_int32 cm_giveUpAllCBs;
+
+extern afs_int32 cm_shutdown;
+#endif /*  OPENAFS_WINNT_AFSD_CM_CALLBACK_H */
